function [grid,P] = Tauchen(rho,N,sigma2, mue)
%TAUCHEN Generates a discrete approximation to an AR-1 Process


sigmaAR = sqrt(sigma2);


grid_probs=linspace(0,1,N+1);   % generate equi-likely bins
bounds=norminv(grid_probs,mue,sigmaAR);     % corresponding bin bounds

% Calculate grid - centers
grid = sigmaAR*N*( normpdf((bounds(1:end-1)-mue)/sigmaAR) - normpdf((bounds(2:end)-mue)/sigmaAR) )+mue;

% replace (-)Inf bounds, by finite numbers
bounds(1)=bounds(2)-99999999999;
bounds(end)=bounds(end-1)+99999999999;

sigma_e=sqrt((1-rho^2)*sigma2);% Calculate short run std
P=zeros(N,N); % Initialize Transition Probability Matrix

for i=1:N %state
    if i ==1
        grid_w = linspace(mue-3*sigmaAR,bounds(i+1),1000)';
        grid_wp = normpdf(grid_w,mue,sigmaAR); %probability to be at these points
        grid_wp = grid_wp./sum(grid_wp);
    elseif i==N
        grid_w = linspace(bounds(i),mue+3*sigmaAR,1000)';
        grid_wp = normpdf(grid_w,mue,sigmaAR); %probability to be at these points
        grid_wp = grid_wp./sum(grid_wp);
    else
        grid_w = linspace(bounds(i),bounds(i+1),1000)';
        grid_wp = normpdf(grid_w,mue,sigmaAR); %probability to be at these points
        grid_wp = grid_wp./sum(grid_wp);
    end
    for j=1:N %transition to
        pi = normcdf(bounds(j+1),rho*grid_w+(1-rho)*mue,sigma_e)-normcdf(bounds(j),rho*grid_w+(1-rho)*mue,sigma_e);
        P(i,j)= grid_wp'*pi;
%         normcdf(bounds(j+1),rho*grid(i)+(1-rho)*mue,sigma_e)-normcdf(bounds(j),rho*grid(i)+(1-rho)*mue,sigma_e)
%         P2(i,j)=normcdf((bounds(j+1)-rho*grid(i)-(1-rho)*mue)/sigma_e)-normcdf((bounds(j)-rho*grid(i)-(1-rho)*mue)/sigma_e);
    end
end

% Make sure P is a Probability Matrix
ps=sum(P,2);
P=P./(repmat(ps,1,N));
        
end