clear all
close all
clc


%Business cycle statistics

ERTE_available = 1;
short_recession = 1;
%% Parameters

if short_recession==0
    TEND = 12*7;
    T1 = 12*5; %length of recession
else
    TEND = 12*3.5;
    T1 = 12*1.5; %length of recession
end

%discounting
freq = 12; %frequency monthly 
dis = 0.96^(1/freq); % Discount factor
mu = 1/(45*freq); %death probability 
mu1 = 1-mu; %survival probability
flex = 1/10; %probability to go to flexible wage

%log productivity of sectors
z = [0.026; 0.0]; %highly affected and weakly affected sector  
zt = [-0.170; -0.048]; %size of business cycle shock in H and W sector 
nz = 2; %number of states
z = [z z+zt];


%distribution of idiosyncratic match qualities    
N_xi = 9; %number of match states
xi_std = 0.21; 
rho_xi = 0.85; %average duration of ERTE?
[xi,Pxi] = Tauchen(rho_xi,N_xi,xi_std^2,0); 
xi_prob = 1/N_xi*ones(N_xi,1);
xi = xi';
 
Pxi_tran = cumsum(Pxi,2);   

%job destruction process
delta = [0.0230; 0.0225]; %exogenous job destruction rate 
delta1 = 1-delta; %probability of job not being exogenously destroyed

%skill parameters
p_e = 13/(45*freq); %learning by doing probability  
p_u = 0.00; %skill loss probability

%skill grids
N_x = 13; %number of worker states
mue = 7.3; %log mean of the distribution 
x_max = mue+0.11; 
x_min = mue-(x_max-mue);
x = linspace(x_min,x_max,N_x)';
x = repmat(x,[1 2]); %for the two sectors 

%preference grid
N_pr = 18; %number of worker states 
std_pr = 34; %standard deviation of distribution    
mue_pr = -56; %mean of the distribution 
[pr_grid,pr_prob,pr_bounds] = Norm(N_pr,mue_pr,std_pr); 
varphi = 1; %probability that preferences are redrawn  
tran_pr_new = (1-varphi)*eye(N_pr,N_pr)+varphi*repmat(pr_prob',[N_pr 1]); %implied transition matrix
tran_pr_new = tran_pr_new./repmat(sum(tran_pr_new,2),[1 N_pr]); %make sure probabilities sum to 1

prob_pr_birth = pr_prob;

%matching function
chi = [0.60; 0.05]; %scaling parameter depending on employment state 
gamma = 0.5; %elasticity of those searching

psi = [0 0]; %firing costs
fc_W = 138; %fixed cost of operation in W
fc_H = (1439/1412)*fc_W;
eta = [0.0367*1439+0.045*(1439/0.90); 0.0367*1412+0.045*(1412/0.90)]*3; %vacancy posting costs in H and W
if ERTE_available==1
    kappa = 15.8;
    kappa = [kappa; 1412/1439*kappa]; %costs to firms in ERTE
else
    kappa = [100000; 100000]; %costs to firms in ERTE
end
b_U = 0.58*(0.73*1412+0.27*1439); %unemployment benefits


%output
y_H = exp(repmat(reshape(z(1,:),[1 1 1 nz]),[N_x N_x N_xi 1])+repmat(x(:,1),[1 N_x N_xi nz])+repmat(reshape(xi,[1 1 N_xi 1]),[N_x N_x 1 nz])); %output in H
y_W = exp(repmat(reshape(z(2,:),[1 1 1 nz]),[N_x N_x N_xi 1])+repmat(x(:,2)',[N_x 1 N_xi nz])+repmat(reshape(xi,[1 1 N_xi 1]),[N_x N_x 1 nz])); %output in W

%wage
w_H = 0.90*y_H; 
w_W = 0.90*y_W;


b_RH = 0.7*1439*ones(size(w_H)); %ERTE benefits
b_RW = 0.7*1412*ones(size(w_H));


y_H_L = repmat(y_H,[1 1 1 1 N_pr]);
y_W_L = repmat(y_W,[1 1 1 1 N_pr]);
w_H_L = repmat(w_H,[1 1 1 1 N_pr]);
w_W_L = repmat(w_W,[1 1 1 1 N_pr]);
b_RH_L = repmat(b_RH,[1 1 1 1 N_pr]);
b_RW_L = repmat(b_RW,[1 1 1 1 N_pr]);
pr_grid_L = repmat(reshape(pr_grid,[1 1 1 N_pr]),[N_x N_x N_xi]);


%% offer matrices F

offer_U = repmat(reshape(xi_prob,[1 1 N_xi]),[N_x N_x 1 N_pr]); %match quality offer probability for unemployed workers (this is independnet of their current states)
offer_R = repmat(reshape(xi_prob,[1 1 1 1 N_xi]),[N_x N_x N_xi N_pr]); %match quality offer probability for workers on ERTE (this is independent of their current states)

%% skill transition matrices for linear interpolation

id = zeros(N_x,N_x); %current linear ID in 2-D grid
for k = 1:N_x
    id(:,k) = N_x*(k-1)+(1:N_x)'; 
end

index_H_1 = zeros(N_x,N_x); %next index in 2D matrix in H dimension
for k = 1:N_x
    index_H_1(:,k) = N_x*(k-1)+min((1:N_x)+1,N_x);
end

skill.emp_H.id = id;
skill.emp_H.index_H_1 = index_H_1;


index_W_1 = zeros(N_x,N_x); %next index in 2D matrix in W dimension
for k = 1:N_x
    index_W_1(:,k) = N_x*(min(k,N_x-1))+(1:N_x);
end

skill.emp_W.id = id;
skill.emp_H.index_W_1 = index_W_1;


%% In the last period of simulation, the economy is back to steady state

addpath('D:\Sectors\Clean\Sensitivity') 
addpath('D:\Sectors\Clean\Sensitivity') 

load('steady.mat','steady')

steady.J_RH = -999999999*ones(N_x,N_x,N_xi,N_pr);
steady.J_RW = -999999999*ones(N_x,N_x,N_xi,N_pr);
steady.R_H = -999999999*ones(N_x,N_x,N_xi,N_pr);
steady.R_W = -999999999*ones(N_x,N_x,N_xi,N_pr);

J_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
J_H(:,:,:,:,end) = steady.J_H;
J_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
J_W(:,:,:,:,end) = steady.J_W;
J_RH = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
J_RH(:,:,:,:,end) = steady.J_RH;
J_RW = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
J_RW(:,:,:,:,end) = steady.J_RW;
E_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
E_H(:,:,:,:,end) = steady.E_H;
E_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
E_W(:,:,:,:,end) = steady.E_W;
R_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
R_W(:,:,:,:,end) = steady.R_W;
R_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
R_H(:,:,:,:,end) = steady.R_H;
U = zeros(N_x,N_x,N_pr,TEND+1);
U(:,:,:,end) = steady.U;

p_W_U = zeros(N_x,N_x,N_pr,TEND+1);
p_W_U(:,:,:,end) = steady.p_W_U;
p_H_U = zeros(N_x,N_x,N_pr,TEND+1);
p_H_U(:,:,:,end) = steady.p_H_U;
p_H_RH = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
p_H_RH(:,:,:,:,end) = steady.p_H_RH;
p_W_RH = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
p_W_RH(:,:,:,:,end) = steady.p_W_RH;
p_W_RW = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
p_W_RW(:,:,:,:,end) = steady.p_W_RW;
p_H_RW = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
p_H_RW(:,:,:,:,end) = steady.p_H_RW;

pol_U_HW = zeros(N_x,N_x,N_pr,TEND+1);
pol_U_HW(:,:,:,end) = steady.pol_U_HW;
pol_U_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_U_W(:,:,:,:,end) = steady.pol_U_W;
pol_U_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_U_H(:,:,:,:,end) = steady.pol_U_H;
pol_F_fill_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_fill_W(:,:,:,:,end) = steady.pol_F_fill_W;
pol_F_fill_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_fill_H(:,:,:,:,end) = steady.pol_F_fill_H;
pol_F_H_N = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_H_N(:,:,:,:,end) = steady.pol_F_H_N;
pol_F_W_N = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_W_N(:,:,:,:,end) = steady.pol_F_W_N;
pol_F_RH_U = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_RH_U(:,:,:,:,end) = steady.pol_F_RH_U;
pol_F_RW_U = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_RW_U(:,:,:,:,end) = steady.pol_F_RW_U;
pol_F_RH_H = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_RH_H(:,:,:,:,end) = steady.pol_F_RH_H;
pol_F_RW_W = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_F_RW_W(:,:,:,:,end) = steady.pol_F_RW_W;
pol_RW_s = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_RW_s(:,:,:,:,end) = steady.pol_RW_s;
pol_RH_s = zeros(N_x,N_x,N_xi,N_pr,TEND+1);
pol_RH_s(:,:,:,:,end) = steady.pol_RH_s;
pol_RH_H = zeros(N_x,N_x,N_xi,N_pr,N_xi,TEND+1);
pol_RH_H(:,:,:,:,:,end) = steady.pol_RH_H;
pol_RH_W = zeros(N_x,N_x,N_xi,N_pr,N_xi,TEND+1);
pol_RH_W(:,:,:,:,:,end) = steady.pol_RH_W;
pol_RW_W = zeros(N_x,N_x,N_xi,N_pr,N_xi,TEND+1);
pol_RW_W(:,:,:,:,:,end) = steady.pol_RW_W;
pol_RW_H = zeros(N_x,N_x,N_xi,N_pr,N_xi,TEND+1);
pol_RW_H(:,:,:,:,:,end) = steady.pol_RW_H;



%% Main routine


t = 0; %time index
while t<TEND
    t = t+1;

    %% compute expected values 
    
    
    EU = U(:,:,:,end-t+1);
    EE_H1 = E_H(:,:,:,:,end-t+1);
    EE_W1 = E_W(:,:,:,:,end-t+1);
    ER_H1 = R_H(:,:,:,:,end-t+1);
    ER_W1 = R_W(:,:,:,:,end-t+1);
    EJ_H1 = J_H(:,:,:,:,end-t+1);
    EJ_W1 = J_W(:,:,:,:,end-t+1);
    EJ_RH1 = J_RH(:,:,:,:,end-t+1);
    EJ_RW1 = J_RW(:,:,:,:,end-t+1);

    %expectations over idiosyncratic match productivity
    EE_H = zeros(N_x,N_x,N_xi,N_pr);
    EE_W = zeros(N_x,N_x,N_xi,N_pr);
    ER_H = zeros(N_x,N_x,N_xi,N_pr);
    ER_W = zeros(N_x,N_x,N_xi,N_pr);
    EJ_H = zeros(N_x,N_x,N_xi,N_pr);
    EJ_W = zeros(N_x,N_x,N_xi,N_pr);
    EJ_RH = zeros(N_x,N_x,N_xi,N_pr);
    EJ_RW = zeros(N_x,N_x,N_xi,N_pr);
    for ix = 1:N_xi %state
        for jx = 1:N_xi %transition to
            EE_H(:,:,ix,:) = EE_H(:,:,ix,:)+Pxi(ix,jx)*EE_H1(:,:,jx,:);
            EE_W(:,:,ix,:) = EE_W(:,:,ix,:)+Pxi(ix,jx)*EE_W1(:,:,jx,:);
            ER_H(:,:,ix,:) = ER_H(:,:,ix,:)+Pxi(ix,jx)*ER_H1(:,:,jx,:);
            ER_W(:,:,ix,:) = ER_W(:,:,ix,:)+Pxi(ix,jx)*ER_W1(:,:,jx,:);
            EJ_H(:,:,ix,:) = EJ_H(:,:,ix,:)+Pxi(ix,jx)*EJ_H1(:,:,jx,:);
            EJ_W(:,:,ix,:) = EJ_W(:,:,ix,:)+Pxi(ix,jx)*EJ_W1(:,:,jx,:);
            EJ_RH(:,:,ix,:) = EJ_RH(:,:,ix,:)+Pxi(ix,jx)*EJ_RH1(:,:,jx,:);
            EJ_RW(:,:,ix,:) = EJ_RW(:,:,ix,:)+Pxi(ix,jx)*EJ_RW1(:,:,jx,:);
        end
    end
     
    %expected firm and worker values depending on worker state today (linearly interpolate productivity states)
    for j1 = 1:N_pr
        for i1 = 1:N_xi %match state
            aux = EE_H(:,:,i1,j1);
            EE_H_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value of H being employed today in H
            aux = EE_W(:,:,i1,j1);
            EE_W_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value W being employed today in W
            aux = ER_H(:,:,i1,j1);
            ER_H_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value of RH being employed today in H
            aux = ER_W(:,:,i1,j1);
            ER_W_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value of RW being employed today in W
            aux = EJ_H(:,:,i1,j1);
            EJ_H_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value J in H being employed today in H
            aux = EJ_W(:,:,i1,j1);
            EJ_W_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value J in W being employed today in W
            aux = EJ_RH(:,:,i1,j1);
            EJ_RH_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value JR in H being employed today in H
            aux = EJ_RW(:,:,i1,j1);
            EJ_RW_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value J in W being employed today in W
        end
        aux = EU(:,:,j1);
        EU_H1(:,:,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value U being H today
        EU_W1(:,:,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value U being W today
    end
    
    %expected value of unemployment when coming from employment
    EU_H = zeros(N_x,N_x,N_pr);
    EU_W = zeros(N_x,N_x,N_pr);
    for i = 1:N_pr %state
        for j= 1:N_pr %transition to
            EU_H(:,:,i) = EU_H(:,:,i)+tran_pr_new(i,j)*EU_H1(:,:,j);
            EU_W(:,:,i) = EU_W(:,:,i)+tran_pr_new(i,j)*EU_W1(:,:,j);
        end
    end
    
    %% Compute upper envelopes over expected values and resulting policy functions
    EU_L = repmat(reshape(EU,[N_x N_x 1 N_pr]),[1 1 N_xi 1]); %expand dimension of expected value of unemployment by match states
    EU_H_L = repmat(reshape(EU_H,[N_x N_x 1 N_pr]),[1 1 N_xi 1]); %expand dimension of expected value of unemployment by match states
    EU_W_L = repmat(reshape(EU_W,[N_x N_x 1 N_pr]),[1 1 N_xi 1]);
    
    %firm decision between continuing, layoff, ERTE 
    upper_F_RH_U = max(EJ_RH_H,-psi(1)); %upper envelop between ERTE H firm and layoff 
    upper_F_H_N = max(EJ_H_H,upper_F_RH_U); %upper envelop between continuing a H firm and not
    upper_F_RW_U = max(EJ_RW_W,-psi(2)); %upper envelop between ERTE W firm and layoff
    upper_F_W_N = max(EJ_W_W,upper_F_RW_U); %upper envelop between continuing a W firm and not
    pol_F_RH_U(:,:,:,:,end-t) = -psi(1)>EJ_RH_H; %corresponding policy (prefer layoff)
    pol_F_H_N(:,:,:,:,end-t) = upper_F_RH_U>EJ_H_H; %corresponding policy (prefer not to continue)
    pol_F_RW_U(:,:,:,:,end-t) = -psi(2)>EJ_RW_W; %corresponding policy (prefer layoff)
    pol_F_W_N(:,:,:,:,end-t) = upper_F_RW_U>EJ_W_W; %corresponding policy (prefer not to continue)
    
    %recalling the worker from ERTE decision (part of 5.3)
    upper_F_RH_H = max(EJ_RH,EJ_H); %upper envelop between ERTE H firm and recalling worker 
    upper_F_RW_W = max(EJ_RW,EJ_W); %upper envelop between ERTE W firm and recalling worker
    pol_F_RH_H(:,:,:,:,end-t) = EJ_H>EJ_RH; %corresponding policy (prefer recall)
    pol_F_RW_W(:,:,:,:,end-t) = EJ_W>EJ_RW; %corresponding policy (prefer recall)

    
    %decision of a unemployed searching worker to accept a potential job offer
    pol_U_H(:,:,:,:,end-t) = EE_H>EU_L; %corresponding policy (prefer employment)
    pol_U_W(:,:,:,:,end-t) = EE_W>EU_L; %corresponding policy (prefer employment)

    
    %continuation value in ERTE (equation 5.10)
    Gamma_H = pol_F_RH_H(:,:,:,:,end-t).*EE_H+(1-pol_F_RH_H(:,:,:,:,end-t)).*ER_H;
    Gamma_W = pol_F_RW_W(:,:,:,:,end-t).*EE_W+(1-pol_F_RW_W(:,:,:,:,end-t)).*ER_W;
    
    %decision of a ERTE searching worker to accept a potential job offer 
    aux_GH = repmat(Gamma_H,[1 1 1 1 N_xi]); %value of staying extended by offer grid
    aux_EH = repmat(permute(EE_H,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer value independent of current state
    aux_EW = repmat(permute(EE_W,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer value independent of current state
    aux_GW = repmat(Gamma_W,[1 1 1 1 N_xi]); %extend by offer grid
    pol_RH_H(:,:,:,:,:,end-t) = aux_EH>aux_GH; %corresponding policy (prefer offer)
    pol_RH_W(:,:,:,:,:,end-t) = aux_EW>aux_GH; %corresponding policy (prefer offer)
    pol_RW_W(:,:,:,:,:,end-t) = aux_EW>aux_GW; %corresponding policy (prefer offer)
    pol_RW_H(:,:,:,:,:,end-t) = aux_EH>aux_GW; %corresponding policy (prefer offer)


    % Implied labor market tightness
    aux = max(pol_U_H(:,:,:,:,end-t),0.00001); %assure tightness is finite
    aux_vH = max(EJ_H,0);
    pol_F_fill_H(:,:,:,:,end-t) = EJ_H>0; %policy whether to fill a vacancy
    theta_H_U = squeeze((eta(1)./sum((dis*chi(1)*aux.*aux_vH).*offer_U,3)).^(-1/gamma)); %labor market tightness in H for workers coming from U
    aux = max(pol_U_W(:,:,:,:,end-t),0.00001); %assure tightness is finite
    aux_vW = max(EJ_W,0);
    pol_F_fill_W(:,:,:,:,end-t) = EJ_W>0; %policy whether to fill a vacancy
    theta_W_U = squeeze((eta(2)./sum((dis*chi(1)*aux.*aux_vW).*offer_U,3)).^(-1/gamma)); %labor market tightness in W for workers coming from U
    aux = max(pol_RH_H(:,:,:,:,:,end-t),0.00001); %assure tightness is finite
    aux2 = repmat(permute(aux_vH,[1 2 5 4 3]),[1 1 N_xi 1 1]); %value of the firm is independ on place where worker is right now 
    theta_H_RH = squeeze((eta(1)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in H for workers coming from RH
    aux = max(pol_RW_H(:,:,:,:,:,end-t),0.00001); %assure tightness is finite
    theta_H_RW = squeeze((eta(1)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in H for workers coming from RW
    aux = max(pol_RH_W(:,:,:,:,:,end-t),0.00001); %assure tightness is finite
    aux2 = repmat(permute(aux_vW,[1 2 5 4 3]),[1 1 N_xi 1 1]); %value of the firm is independ on place where worker is right now
    theta_W_RH = squeeze((eta(2)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in W for workers coming from RH
    aux = max(pol_RW_W(:,:,:,:,:,end-t),0.00001); %assure tightness is finite
    theta_W_RW = squeeze((eta(2)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in W for workers coming from RW
    
    % Implied job offer rates 
    p_H_U(:,:,:,end-t) = min(max(chi(1)*theta_H_U.^(1-gamma),0.00001),1); %assure bounded probabilities
    p_H_RH(:,:,:,:,end-t) = min(max(chi(2)*theta_H_RH.^(1-gamma),0.00001),1);
    p_H_RW(:,:,:,:,end-t) = min(max(chi(2)*theta_H_RW.^(1-gamma),0.00001),1);
    p_W_U(:,:,:,end-t) = min(max(chi(1)*theta_W_U.^(1-gamma),0.00001),1);
    p_W_RH(:,:,:,:,end-t) = min(max(chi(2)*theta_W_RH.^(1-gamma),0.00001),1);
    p_W_RW(:,:,:,:,end-t) = min(max(chi(2)*theta_W_RW.^(1-gamma),0.00001),1);
      
    

    %choice of worker in ERTE searching in particular sector (have to recompute upper envelops as need to take into account decision of firm to fill vacancy)
    aux_EH = repmat(permute(pol_F_fill_H(:,:,:,:,end-t).*EE_H+(1-pol_F_fill_H(:,:,:,:,end-t)).*Gamma_H,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer value independent of current state
    upper_RH_H = max(aux_EH,aux_GH); %upper envelop between staying ERTE H and going to H
    upper_RW_H = max(aux_EH,aux_GW); %upper envelop between staying ERTE W and going to H
    aux_EW = repmat(permute(pol_F_fill_W(:,:,:,:,end-t).*EE_W+(1-pol_F_fill_W(:,:,:,:,end-t)).*Gamma_W,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer value independent of current state
    upper_RH_W = max(aux_EW,aux_GH); %upper envelop between staying ERTE H and going to W
    upper_RW_W = max(aux_EW,aux_GW); %upper envelop between staying ERTE W and going to W
    RS_HH = (1-p_H_RH(:,:,:,:,end-t)).*Gamma_H+p_H_RH(:,:,:,:,end-t).*sum(upper_RH_H.*offer_R,5); %value of being in H and searching in H
    RS_HW = (1-p_W_RH(:,:,:,:,end-t)).*Gamma_H+p_W_RH(:,:,:,:,end-t).*sum(upper_RH_W.*offer_R,5); %value of being in H and searching in W
    Lambda_H = max(RS_HH,RS_HW); %equation 5.8
    RS_WH = (1-p_H_RW(:,:,:,:,end-t)).*Gamma_W+p_H_RW(:,:,:,:,end-t).*sum(upper_RW_H.*offer_R,5); %value of being in W and searching in H
    RS_WW = (1-p_W_RW(:,:,:,:,end-t)).*Gamma_W+p_W_RW(:,:,:,:,end-t).*sum(upper_RW_W.*offer_R,5); %value of being in W and searching in W
    Lambda_W = max(RS_WH,RS_WW); %equation 5.8
    pol_RH_s(:,:,:,:,end-t) = RS_HH>RS_HW; %corresponding policy (prefer H over W)
    pol_RW_s(:,:,:,:,end-t) = RS_WW>RS_WH; %corresponding policy (prefer W over H)
    
    % Implied job leaving probability from ERTE
    pi_RH = pol_RH_s(:,:,:,:,end-t).*p_H_RH(:,:,:,:,end-t)+(1-pol_RH_s(:,:,:,:,end-t)).*p_W_RH(:,:,:,:,end-t);
    pi_RW = pol_RW_s(:,:,:,:,end-t).*p_W_RW(:,:,:,:,end-t)+(1-pol_RW_s(:,:,:,:,end-t)).*p_H_RW(:,:,:,:,end-t);
    
    %choice of worker in ERTE between staying in ERTE and unemployment
    upper_LU_H = Lambda_H;
    upper_LU_W = Lambda_W;
    pol_LU_H = ones(N_x,N_x,N_xi,N_pr);
    pol_LU_W = ones(N_x,N_x,N_xi,N_pr);
    
    
    %choice of unemployed between searching in sectors (have to recompute upper envelops as need to take into account decision of firm to fill vacancy)
    upper_U_H = max(pol_F_fill_H(:,:,:,:,end-t).*EE_H+(1-pol_F_fill_H(:,:,:,:,end-t)).*EU_L,EU_L); %upper envelop between staying unemployment and going to H
    upper_U_W = max(pol_F_fill_W(:,:,:,:,end-t).*EE_W+(1-pol_F_fill_W(:,:,:,:,end-t)).*EU_L,EU_L); %upper envelop between staying unemployment and going to W
    US_H = (1-p_H_U(:,:,:,end-t)).*EU+p_H_U(:,:,:,end-t).*squeeze(sum(upper_U_H.*offer_U,3));
    US_W = (1-p_W_U(:,:,:,end-t)).*EU+p_W_U(:,:,:,end-t).*squeeze(sum(upper_U_W.*offer_U,3));
    upper_U_HW = max(US_H,US_W); %upper envelope
    pol_U_HW(:,:,:,end-t) = US_H>US_W; %corresponding policy (prefer H)
    
    
    %Compute XI for workers using firm policies (equation 5.6)
    XI_H = (pol_F_H_N(:,:,:,:,end-t)==1).*(pol_F_RH_U(:,:,:,:,end-t).*EU_H_L+(1-pol_F_RH_U(:,:,:,:,end-t)).*ER_H_H)...
        +(1-pol_F_H_N(:,:,:,:,end-t)).*EE_H_H;
    XI_W = (pol_F_W_N(:,:,:,:,end-t)==1).*(pol_F_RW_U(:,:,:,:,end-t).*EU_W_L+(1-pol_F_RW_U(:,:,:,:,end-t)).*ER_W_W)...
        +(1-pol_F_W_N(:,:,:,:,end-t)).*EE_W_W;
    
    
    %quitting decision of the employed
    upper_XI_U_H = XI_H; %upper envelop between staying H and unemployment 
    upper_XI_U_W = XI_W; %upper envelop between staying W and unemployment 
    pol_XI_U_H=0; %always choose employment
    pol_XI_U_W=0;
    
    %% value functions
    discount = dis*(mu1);
    
    
    if (TEND-t)<0 || (TEND-t)>=T1 %boom wage
        w_aux_H =  squeeze(w_H_L(:,:,:,1,:));
        w_aux_W =  squeeze(w_W_L(:,:,:,1,:));
        b_aux_H = squeeze(b_RH_L(:,:,:,1,:));
        b_aux_W = squeeze(b_RW_L(:,:,:,1,:));
    else
        w_aux_H =  squeeze(w_H_L(:,:,:,2,:));
        w_aux_W =  squeeze(w_W_L(:,:,:,2,:));
        b_aux_H = squeeze(b_RH_L(:,:,:,2,:));
        b_aux_W = squeeze(b_RW_L(:,:,:,2,:));
    end
    if (TEND-t)>=T1 %boom output
        y_aux_H =  squeeze(y_H_L(:,:,:,1,:));
        y_aux_W =  squeeze(y_W_L(:,:,:,1,:));
        kappa_aux = [500000000; 500000000];
    else
        y_aux_H =  squeeze(y_H_L(:,:,:,2,:));
        y_aux_W =  squeeze(y_W_L(:,:,:,2,:));
        kappa_aux = kappa;
    end

    U(:,:,:,end-t) = b_U + discount*upper_U_HW; %value of unemployment 

    R_H(:,:,:,:,end-t) = b_aux_H + discount*(delta(1)*EU_L+(1-delta(1))*upper_LU_H); %value of ERTE in H 
    R_W(:,:,:,:,end-t) = b_aux_W + discount*(delta(2)*EU_L+(1-delta(2))*upper_LU_W); %value of ERTE in W 
    E_H(:,:,:,:,end-t) = w_aux_H + pr_grid_L + discount*(delta(1)*EU_L+(1-delta(1))*upper_XI_U_H); %value of employed in H 
    E_W(:,:,:,:,end-t) = w_aux_W - pr_grid_L + discount*(delta(2)*EU_L+(1-delta(2))*upper_XI_U_W); %value of employed in W 
    J_H(:,:,:,:,end-t) = y_aux_H - w_aux_H -fc_H + discount*(1-delta(1))*(pol_XI_U_H==0).*upper_F_H_N; %value of filled job in H
    J_W(:,:,:,:,end-t) = y_aux_W - w_aux_W -fc_W + discount*(1-delta(2))*(pol_XI_U_W==0).*upper_F_W_N; %value of filled job in W
    J_RH(:,:,:,:,end-t) = -kappa_aux(1) + discount*(1-delta(1))*pol_LU_H.*(1-pi_RH).*upper_F_RH_H; %value of job in ERTE H
    J_RW(:,:,:,:,end-t) = -kappa_aux(2) + discount*(1-delta(2))*pol_LU_W.*(1-pi_RW).*upper_F_RW_W; %value of job in ERTE W 


    pol_F_RH_U(:,:,:,:,end-t) = -psi(1)>EJ_RH_H; %corresponding policy (prefer layoff)
    pol_F_H_N(:,:,:,:,end-t) = upper_F_RH_U>EJ_H_H; %corresponding policy (prefer not to continue)
    pol_F_RW_U(:,:,:,:,end-t) = -psi(2)>EJ_RW_W; %corresponding policy (prefer layoff)
    pol_F_W_N(:,:,:,:,end-t) = upper_F_RW_U>EJ_W_W; %corresponding policy (prefer not to continue)
    pol_F_RH_H(:,:,:,:,end-t) = EJ_H>EJ_RH; %corresponding policy (prefer recall)

    pol_F_RW_W(:,:,:,:,end-t) = EJ_W>EJ_RW; %corresponding policy (prefer recall)
    pol_U_H(:,:,:,:,end-t) = EE_H>EU_L; %corresponding policy (prefer employment)
    pol_U_W(:,:,:,:,end-t) = EE_W>EU_L; %corresponding policy (prefer employment)
    pol_RH_H(:,:,:,:,:,end-t) = aux_EH>aux_GH; %corresponding policy (prefer employment)
    pol_RH_W(:,:,:,:,:,end-t) = aux_EW>aux_GH; %corresponding policy (prefer employment)
    pol_RW_W(:,:,:,:,:,end-t) = aux_EW>aux_GW; %corresponding policy (prefer employment)
    pol_RW_H(:,:,:,:,:,end-t) = aux_EH>aux_GW; %corresponding policy (prefer employment)
    pol_RH_s(:,:,:,:,end-t) = RS_HH>RS_HW; %corresponding policy (prefer H over W)
    pol_RW_s(:,:,:,:,end-t) = RS_WW>RS_WH; %corresponding policy (prefer W over H)
    pol_U_HW(:,:,:,end-t) = US_H>US_W; %corresponding policy (prefer H)
    pol_XI_U_H = 0; %corresponding policy (prefer unemployment)
    pol_XI_U_W = 0; %corresponding policy (prefer unemployment)

        
end



%% simulation of recession

%load steady state distribution
load('panel_ss.mat','panel_ss')

H = length(panel_ss); %number of people

panel_ss(:,1) = 1:H; %each observation is a new person

%shocks
rand('seed',397547);
randn('seed',287569);

die = rand(H,TEND); %surviving
offer = rand(H,TEND); %probability to get job offer 
offer_type_p = rand(H,TEND); %probability to get particular job offer 
destruct = rand(H,TEND); %probability to get unemployment exogenously
tran_xi = rand(H,TEND); %transition for match quality
skill_acc = rand(H,TEND); %probability to get one higher skill
shock_pr_new = rand(H,TEND); %probability to get a new preference
shock_pr = rand(H,TEND); %type of new preference


panel = zeros(H,10,TEND+1); %index, employment (U=0, H=1, W=2, RH=3, RW=4), xH, xW, xi, wage, switches job, taste, period, search in sector
panel(:,:,1) = panel_ss; %first period is the staedy state


panel = simulation_mit_mex(panel,int32(H),int32(TEND),int32(T1),int32(N_x),int32(N_xi),int32(N_pr),die,offer,offer_type_p ...
                    ,destruct,tran_xi,skill_acc,shock_pr_new,shock_pr,xi_prob,prob_pr_birth ...
                    ,mu,p_e,varphi,delta,Pxi_tran,w_H,w_W ...
                    ,p_W_U,p_H_U,p_H_RH,p_W_RH,p_W_RW,p_H_RW ...
                    ,int32(pol_U_HW),int32(pol_U_W),int32(pol_F_fill_W),int32(pol_U_H),int32(pol_F_fill_H) ...
                    ,int32(pol_F_H_N),int32(pol_F_RH_U),int32(pol_F_W_N),int32(pol_F_RW_U) ...
                    ,int32(pol_RH_s),int32(pol_RH_H),int32(pol_F_RH_H),int32(pol_RH_W),int32(pol_RW_s) ...
                    ,int32(pol_RW_W),int32(pol_F_RW_W),int32(pol_RW_H));



panel(:,:,end) = []; %drop last period

%collapse to 2d matrix
panel_new = zeros(H*(TEND),10);
for h = 1:H
    transform = squeeze(panel(h,:,1:end));
    transform = transform';
    panel_new(1+(h-1)*(TEND):h*(TEND),:) = transform;
end
panel = panel_new;
panel(panel(:,9)==0,:) = []; %delete the dead
clear panel_new

%unemployment rate
aux = panel;
aux(:,2) = aux(:,2)==0;
aux = sortrows(aux,9);
fcn = @(x){create_rate(x)};
time.urate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %unemployment rate

%working rate
aux = panel;
aux(:,2) = (aux(:,2)==1)+(aux(:,2)==2);
aux = sortrows(aux,9);
fcn = @(x){create_rate(x)};
time.wrate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %unemployment rate

%number of working in H
aux = panel;
aux(:,2) = aux(:,2)==1;
aux = sortrows(aux,9);
fcn = @(x){sum(x)};
time.N_H = accumarray_expand_s(aux(:,2),aux(:,9),fcn); 
fcn = @(x){create_rate(x)};
time.N_H_rate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %H to pop
time.N_H_1 = time.N_H./(time.N_H(1)); %time.N_H_1(56)
if short_recession == 0 
    cali.H_drop = time.N_H_1(56);
    cali.H_drop = mean(time.N_H_1);
end

%number of working in W
aux = panel;
aux(:,2) = aux(:,2)==2;
aux = sortrows(aux,9);
fcn = @(x){sum(x)};
time.N_W = accumarray_expand_s(aux(:,2),aux(:,9),fcn); 
fcn = @(x){create_rate(x)};
time.N_W_rate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %W to pop
time.N_W_1 = time.N_W./(time.N_W(1)); %time.N_W_1(56)
if short_recession == 0 
    cali.W_drop = time.N_W_1(56);
    cali.W_drop = mean(time.N_W_1);
end

%number of people and share of population in ERTE H
aux = panel;
aux(:,2) = aux(:,2)==3;
aux = sortrows(aux,9);
fcn = @(x){sum(x)};
time.N_RH = accumarray_expand_s(aux(:,2),aux(:,9),fcn); 
fcn = @(x){create_rate(x)};
time.RH_rate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %RH to pop

%number of people and share of population in ERTE W
aux = panel;
aux(:,2) = aux(:,2)==4;
aux = sortrows(aux,9);
fcn = @(x){sum(x)};
time.N_RW = accumarray_expand_s(aux(:,2),aux(:,9),fcn);
fcn = @(x){create_rate(x)};
time.RW_rate = accumarray_expand_s(aux(:,2),aux(:,9),fcn); %RW to pop

%share of population in ERTE
time.R = time.RH_rate+time.RW_rate;

stats.share_ERTE = (time.N_RW(:)+time.N_RH(:))./(time.N_RW(:)+time.N_RH(:)+time.N_W(:)+time.N_H(:)); %share of ERTE 
if short_recession == 0 
    cali.share_ERTE = max(stats.share_ERTE);
    cali.share_ERTE = stats.share_ERTE(4);
end

%aggregate output and mean productivity of the employed
aux = panel;
% aux = aux(aux(:,2)<3,:); %drop ERTEs
aux(aux(:,2)==0,:) = []; %drop unemployed
for i = 1:length(aux)
    if aux(i,9)==1 || aux(i,9)>T1 %before or after recession
        if aux(i,2)==1 || aux(i,2)==3 %employed in H
            aux(i,6) = y_H(aux(i,3),aux(i,4),aux(i,5),1);
        else
            aux(i,6) = y_W(aux(i,3),aux(i,4),aux(i,5),1);
        end
    else
        if aux(i,2)==1 || aux(i,2)==3 %employed in H
            aux(i,6) = y_H(aux(i,3),aux(i,4),aux(i,5),2);
        else
            aux(i,6) = y_W(aux(i,3),aux(i,4),aux(i,5),2);
        end
    end
end
aux = sortrows(aux,9);
fcn = @(x){mean(x)};
time.prod = accumarray_expand_s(aux(:,6),aux(:,9),fcn); %average productivity of those employed (includes ERTE)
time.prod_1 = time.prod./(time.prod(1));
aux = aux(aux(:,2)<3,:); %drop ERTEs
aux = sortrows(aux,9);
fcn = @(x){sum(x)};
time.y = accumarray_expand_s(aux(:,6),aux(:,9),fcn); %total output of those working
time.y_1 = time.y./(time.y(1));

%mean H skill in the W sector
aux = panel;
index = (aux(:,2)==2)+(aux(:,2)==4);
aux = aux(index==1,:); %currently in W
aux(:,3) = x(aux(:,3),1); %translate grid points to values
aux = sortrows(aux,9);
fcn = @(x){mean(x)};
time.H_in_W = accumarray_expand_s(aux(:,3),aux(:,9),fcn); 
time.H_in_W_1 = time.H_in_W./(time.H_in_W(1));

%Relative size of H sector
aux = panel;
aux(aux(:,2)==0,:) = []; %drop unemployed
aux(:,10) = (aux(:,2)==1)+(aux(:,2)==3); %currently in H
aux = sortrows(aux,9);
fcn = @(x){create_rate(x)};
time.size_H = accumarray_expand_s(aux(:,10),aux(:,9),fcn); 

%share of unemployed searching in H 
aux = panel;
aux(:,11) = aux(:,2)>0;%single employment index
aux = aux(aux(:,11)==0,:);
aux(:,10) = aux(:,10)==1; %search in H
fcn = @(x){create_rate(x)};
aux = sortrows(aux,9);
time.search_H = accumarray_expand_s(aux(:,10),aux(:,9),fcn); 

%probability that a worker in ERTE is in different firm in t+12
aux = panel;
fcn = @(x){create_LEAD12(x)};
aux(:,10) = accumarray_expand(aux(:,2),aux(:,1),fcn); %next period's employment state
aux(isnan(aux(:,10))==1,10) = 0; %count dead as unemployed
fcn = @(x){sum12(x)};
aux(:,11) = accumarray_expand(aux(:,7),aux(:,1),fcn); %switched jobs in the next 12 periods
aux(:,11) = aux(:,11)>0; %make it an indicator
aux(aux(:,2)<3,:) = []; %keep only those currently on ERTE
aux(isnan(aux(:,10)),:) = []; %delete those without emp info in t+12
index = (aux(:,11)==1).*(aux(:,10)>0); %switched jobs and employed in t+12
stats.share_ERTE_switch = sum(index)/length(index);
aux2 = aux(aux(:,2)==3,:); %currently in H
index = (aux2(:,11)==1).*(aux2(:,10)>0); %unemployed in t+12
stats.share_ERTE_H_switch = sum(index)/length(index);
aux2 = aux(aux(:,2)==4,:); %currently in W
index = (aux2(:,11)==1).*(aux2(:,10)>0); %unemployed in t+12
stats.share_ERTE_W_switch = sum(index)/length(index);
if short_recession == 0 
    cali.share_ERTE_switch = stats.share_ERTE_switch;
end

%probability that a worker in ERTE is in same firm in t+12
aux = panel;
fcn = @(x){create_LEAD12(x)};
aux(:,10) = accumarray_expand(aux(:,2),aux(:,1),fcn); %next period's employment state
aux(isnan(aux(:,10))==1,10) = 0; %count dead as unemployed
fcn = @(x){sum12(x)};
aux(:,11) = accumarray_expand(aux(:,7),aux(:,1),fcn); %switched jobs in the next 12 periods
aux(:,11) = aux(:,11)>0; %make it an indicator
aux(aux(:,2)<3,:) = []; %keep only those currently on ERTE
aux(isnan(aux(:,10)),:) = []; %delete those without emp info in t+12
index = (aux(:,11)==0).*(aux(:,10)>0); %did not switch jobs and still employed
stats.share_ERTE_stay = sum(index)/length(index);
aux2 = aux(aux(:,2)==3,:); %currently in H
index = (aux2(:,11)==0).*(aux2(:,10)>0); %unemployed in t+12
stats.share_ERTE_H_stay = sum(index)/length(index);
aux2 = aux(aux(:,2)==4,:); %currently in W
index = (aux2(:,11)==0).*(aux2(:,10)>0); %unemployed in t+12
stats.share_ERTE_W_stay = sum(index)/length(index);
if short_recession == 0 
    cali.share_ERTE_stay = stats.share_ERTE_stay;
end

%probability that a worker in ERTE is unemployed in t+12
aux = panel;
fcn = @(x){create_LEAD12(x)};
aux(:,10) = accumarray_expand(aux(:,2),aux(:,1),fcn); %next period's employment state
aux(aux(:,2)<3,:) = []; %keep only those currently on ERTE
aux(isnan(aux(:,10)),:) = []; %delete those without emp info in t+12
index = (aux(:,10)==0); %unemployed in t+12
stats.share_ERTE_u = sum(index)/length(index);
aux2 = aux(aux(:,2)==3,:); %currently in H
index = (aux2(:,10)==0); %unemployed in t+12
stats.share_ERTE_H_u = sum(index)/length(index);
aux2 = aux(aux(:,2)==4,:); %currently in W
index = (aux2(:,10)==0); %unemployed in t+12
stats.share_ERTE_W_u = sum(index)/length(index);

%UE transition rate
aux = panel;
aux(:,10) = aux(:,2)>0; %single employment index
fcn = @(x){create_diff(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
fcn = @(x){create_LEAD(x)};
aux(:,12) = accumarray_expand(aux(:,2),aux(:,1),fcn); %next period's employment state
aux = aux(aux(:,2)==0,:); %keep the unemployed
aux(isnan(aux(:,11)),:) = []; %delete last observation
stats.UE = sum(aux(:,11)==1)./length(aux); %unconditional probability to find a job
aux(:,13) = cumsum(aux(:,12)); %unemployment spell
aux(2:end,13) =aux(1:end-1,13);
fcn = @(x){max(x)};
aux(:,12) = accumarray_expand(aux(:,12),aux(:,13),fcn); %copy next employment type to the entire unemploeymrnt spell
aux2 = aux(aux(:,12)==1,:); %unemployed who end up in sector H
stats.UE_H = sum(aux2(:,11)==1)./length(aux2);
aux2 = aux(aux(:,12)==2,:); %unemployed who end up in sector W
stats.UE_W = sum(aux2(:,11)==1)./length(aux2);


load('moments.mat','moments')
if short_recession==0
    if ERTE_available==0
        moments.noERTE.stats = stats;
        moments.noERTE.time = time;
    else
        moments.ERTE.stats = stats;
        moments.ERTE.time = time;
    end
else
    if ERTE_available==0
        moments.short.noERTE.stats = stats;
        moments.short.noERTE.time = time;
    else
        moments.short.ERTE.stats = stats;
        moments.short.ERTE.time = time;
    end
end
save('moments.mat','moments')


ccc


%unemployment rate with and without ERTE
figure(1)
plot(moments.noERTE.time.urate-(moments.noERTE.time.urate(1)),'LineWidth',2.5,'Color',[0 0 1])
hold on
plot(moments.ERTE.time.urate-(moments.ERTE.time.urate(1)),'LineWidth',2.5,'LineStyle','--','Color',[1 0 0])
hold off
ax = gca;
ax.FontSize = 16; 
xlabel('Month','fontsize',16)
ylabel('Unemployment rate','fontsize',16)
width=350;
height=350;
set(gcf,'units','points','position',[0,0,width,height])
xlim([1 6*12])
ylim([-0.035 0.25])
legend('No ERTE','ERTE','FontSize',13)


%output with and without ERTE
figure(2)
plot(moments.noERTE.time.y_1-1,'LineWidth',2.5,'Color',[0 0 1])
hold on
plot(moments.ERTE.time.y_1-1,'LineWidth',2.5,'LineStyle','--','Color',[1 0 0])
hold off
ax = gca;
ax.FontSize = 16; 
xlabel('Month','fontsize',16)
ylabel('Output','fontsize',16)
width=350;
height=350;
set(gcf,'units','points','position',[0,0,width,height])
xlim([1 6*12])
ylim([-0.35 0])
legend('No ERTE','ERTE','FontSize',13)



%unemployment rate with and without ERTE
figure(3)
plot(moments.short.noERTE.time.urate-(moments.short.noERTE.time.urate(1)),'LineWidth',2.5,'Color',[0 0 1])
hold on
plot(moments.short.ERTE.time.urate-(moments.short.ERTE.time.urate(1)),'LineWidth',2.5,'LineStyle','--','Color',[1 0 0])
hold off
ax = gca;
ax.FontSize = 16; 
xlabel('Month','fontsize',16)
ylabel('Unemployment rate','fontsize',16)
width=350;
height=350;
set(gcf,'units','points','position',[0,0,width,height])
xlim([1 2.5*12])
ylim([-0.035 0.15])
legend('No ERTE','ERTE','FontSize',13)


%output with and without ERTE
figure(4)
plot(moments.short.noERTE.time.y_1-1,'LineWidth',2.5,'Color',[0 0 1])
hold on
plot(moments.short.ERTE.time.y_1-1,'LineWidth',2.5,'LineStyle','--','Color',[1 0 0])
hold off
ax = gca;
ax.FontSize = 16; 
xlabel('Month','fontsize',16)
ylabel('Output','fontsize',16)
width=350;
height=350;
set(gcf,'units','points','position',[0,0,width,height])
xlim([1 2.5*12])
ylim([-0.30 0])
legend('No ERTE','ERTE','FontSize',13)


