clear all
close all
clc

%steady state model without ERTE


%% Parameters


%discounting
freq = 12; %frequency monthly 
dis = 0.96^(1/freq); % Discount factor
mu = 1/(45*freq); %death probability
mu1 = 1-mu; %survival probability
 
%log productivity of sectors
z = [0.024; 0.0]; %highly affected and weakly affected sector  


%distribution of idiosyncratic match qualities    
N_xi = 9; %number of match states
xi_std = 0.22; %standard deviation
rho_xi = 0.85; %persistence
[xi,Pxi] = Tauchen(rho_xi,N_xi,xi_std^2,0); 
xi_prob = 1/N_xi*ones(N_xi,1);
xi = xi';
 
Pxi_tran = cumsum(Pxi,2); %CDF  

%job destruction process
delta = [0.0200; 0.0195]; %exogenous job destruction rate 
delta1 = 1-delta; %probability of job not being exogenously destroyed

%skill parameters
p_e = 13/(45*freq); %learning by doing probability  
p_u = 0.00; %skill loss probability

%skill grids
N_x = 13; %number of worker states
mue = 7.235; %log mean of the distribution 
x_max = mue+0.15; %maximum value
x_min = mue-(x_max-mue); %minimum value
x = linspace(x_min,x_max,N_x)'; %grid
x = repmat(x,[1 2]); %for the two sectors 

%preference grid
N_pr = 18; %number of worker states 
std_pr = 36; %standard deviation of distribution  
mue_pr = -72; %mean of the distribution 
[pr_grid,pr_prob,pr_bounds] = Norm(N_pr,mue_pr,std_pr); 
varphi = 1; %probability that preferences are redrawn  
tran_pr_new = (1-varphi)*eye(N_pr,N_pr)+varphi*repmat(pr_prob',[N_pr 1]); %implied transition matrix
tran_pr_new = tran_pr_new./repmat(sum(tran_pr_new,2),[1 N_pr]); %make sure probabilities sum to 1

prob_pr_birth = pr_prob;

%matching function
chi = [1.05; 0.1]; %scaling parameter depending on employment state 
gamma = 0.5; %elasticity of those searching

psi = [0 0];  %firing costs 
fc_W = 64.0; %fixed cost of operation in W
fc_H = (1439/1412)*fc_W; 
eta = [0.0367*1439+0.045*(1439/0.95); 0.0367*1412+0.045*(1412/0.95)]*3; %vacancy posting costs in H and W   
kappa = [500000000; 500000000]; %costs to firms in ERTE 
b_U = 0.58*(0.73*1412+0.27*1439); %unemployment benefits


%output 
y_H = exp(z(1)+repmat(x(:,1),[1 N_x N_xi])+repmat(reshape(xi,[1 1 N_xi 1]),[N_x N_x 1])); %output in H
y_W = exp(z(2)+repmat(x(:,2)',[N_x 1 N_xi])+repmat(reshape(xi,[1 1 N_xi 1]),[N_x N_x 1])); %output in W

%wages
w_H = 0.95*y_H; 
w_W = 0.95*y_W;

b_RH = 0.7*w_H; %ERTE benefits
b_RW = 0.7*w_W;

%span income variables over state space
y_H_L = repmat(y_H,[1 1 1 N_pr]);
y_W_L = repmat(y_W,[1 1 1 N_pr]);
w_H_L = repmat(w_H,[1 1 1 N_pr]);
w_W_L = repmat(w_W,[1 1 1 N_pr]);
b_RH_L = repmat(b_RH,[1 1 1 N_pr]);
b_RW_L = repmat(b_RW,[1 1 1 N_pr]);
pr_grid_L = repmat(reshape(pr_grid,[1 1 1 N_pr]),[N_x N_x N_xi]);

%% offer matrices F

offer_U = repmat(reshape(xi_prob,[1 1 N_xi]),[N_x N_x 1 N_pr]); %match quality offer probability for unemployed workers 
offer_R = repmat(reshape(xi_prob,[1 1 1 1 N_xi]),[N_x N_x N_xi N_pr]); %match quality offer probability for workers on ERTE

%% skill transition matrices for linear interpolation

id = zeros(N_x,N_x); %current linear ID in 2-D grid
for k = 1:N_x
    id(:,k) = N_x*(k-1)+(1:N_x)'; 
end

index_H_1 = zeros(N_x,N_x); %next index in 2D matrix in H dimension
for k = 1:N_x
    index_H_1(:,k) = N_x*(k-1)+min((1:N_x)+1,N_x);
end

skill.emp_H.id = id;
skill.emp_H.index_H_1 = index_H_1;


index_W_1 = zeros(N_x,N_x); %next index in 2D matrix in W dimension
for k = 1:N_x
    index_W_1(:,k) = N_x*(min(k,N_x-1))+(1:N_x);
end

skill.emp_W.id = id;
skill.emp_H.index_W_1 = index_W_1;


%% Initial guess for the value functions


%guess value functions
E_H = dis/(1-dis)*w_H_L; %value of employment in H
E_W = dis/(1-dis)*w_W_L; %value of employment in W
U = dis/(1-dis)*b_U*ones(N_x,N_x,N_pr); %Value of unemployment
J_H = dis/(1-dis)*(y_H_L-w_H_L-fc_H); %value of a firm in H
J_W = dis/(1-dis)*(y_W_L-w_W_L-fc_W); %value of a firm in W

p = min(chi.*([0.4; 1].^(1-gamma)),1); %job-finding probability
q = chi.*([0.4; 1].^(-gamma)); %vacancy-filling probability

%compute value function given simple decision rules
crit = 100000;
while crit>100
    %expected value functions over business cycle state
    EU = U;
    EE_H1 = E_H;
    EE_W1 = E_W;
    EJ_H1 = J_H;
    EJ_W1 = J_W;

    %expected value functions over match quality state
    EE_H = zeros(N_x,N_x,N_xi,N_pr);
    EE_W = zeros(N_x,N_x,N_xi,N_pr);
    EJ_H = zeros(N_x,N_x,N_xi,N_pr);
    EJ_W = zeros(N_x,N_x,N_xi,N_pr);
    for ix = 1:N_xi %state
        for jx = 1:N_xi %transition to
            EE_H(:,:,ix,:) = EE_H(:,:,ix,:)+Pxi(ix,jx)*EE_H1(:,:,jx,:);
            EE_W(:,:,ix,:) = EE_W(:,:,ix,:)+Pxi(ix,jx)*EE_W1(:,:,jx,:);
            EJ_H(:,:,ix,:) = EJ_H(:,:,ix,:)+Pxi(ix,jx)*EJ_H1(:,:,jx,:);
            EJ_W(:,:,ix,:) = EJ_W(:,:,ix,:)+Pxi(ix,jx)*EJ_W1(:,:,jx,:);
        end
    end
    
    EUL = repmat(reshape(EU,[N_x N_x 1 N_pr]),[1 1 N_xi 1]); %span the expected value of unemployment by idiosyncratic match productivity state
    Enew_H = w_H_L+pr_grid_L+dis*(mu1)*(delta1(1)*EE_H+delta(1)*EUL);
    Enew_W = w_W_L-pr_grid_L+dis*(mu1)*(delta1(2)*EE_W+delta(2)*EUL);
    U_H = b_U+dis*(mu1)*((1-p(1)).*EU+p(1).*squeeze(sum(offer_U.*EE_H,3)));
    U_W = b_U+dis*(mu1)*((1-p(2)).*EU+p(2).*squeeze(sum(offer_U.*EE_W,3)));
    U_new = max(U_H,U_W);
    Jnew_H = y_H_L-w_H_L-fc_H+dis*(mu1)*delta1(1)*EJ_H;
    Jnew_W = y_W_L-w_W_L-fc_W+dis*(mu1)*delta1(2)*EJ_W;

    crit = max([max(max(max(max(max(abs(Enew_H-E_H)))))) max(max(max(max(max(abs(Enew_W-E_W))))))...
        max(max(max(max(max(abs(Jnew_H-J_H)))))) max(max(max(max(max(abs(Jnew_W-J_W)))))) max(max(max(max(max(abs(U_new-U))))))]);
    E_H = Enew_H;
    E_W = Enew_W;
    U = U_new;
    J_H = Jnew_H;
    J_W = Jnew_W;
    
end


%% Main routine



count_q = 0; %used to update policy function q
count_first =1; %first iteration

count_policy = 0; %used for policy update
crit = 500;
while crit>150 
    count_policy = count_policy+1;
    %% compute expected values 
    
    %expectations over business cycle state (steady state)
    EU = U;
    EE_H1 = E_H;
    EE_W1 = E_W;
    EJ_H1 = J_H;
    EJ_W1 = J_W;
    
    %expectations over idiosyncratic match productivity
    EE_H = zeros(N_x,N_x,N_xi,N_pr);
    EE_W = zeros(N_x,N_x,N_xi,N_pr);
    EJ_H = zeros(N_x,N_x,N_xi,N_pr);
    EJ_W = zeros(N_x,N_x,N_xi,N_pr);
    for ix = 1:N_xi %state
        for jx = 1:N_xi %transition to
            EE_H(:,:,ix,:) = EE_H(:,:,ix,:)+Pxi(ix,jx)*EE_H1(:,:,jx,:);
            EE_W(:,:,ix,:) = EE_W(:,:,ix,:)+Pxi(ix,jx)*EE_W1(:,:,jx,:);
            EJ_H(:,:,ix,:) = EJ_H(:,:,ix,:)+Pxi(ix,jx)*EJ_H1(:,:,jx,:);
            EJ_W(:,:,ix,:) = EJ_W(:,:,ix,:)+Pxi(ix,jx)*EJ_W1(:,:,jx,:);
        end
    end
    ER_H = zeros(N_x,N_x,N_xi,N_pr); %for ERTE. There are no ERTEs here, so vlue and policy functions will simply be 0
    ER_W = zeros(N_x,N_x,N_xi,N_pr);
     
    %expected firm and worker values over productivity development depending on worker state today (linearly interpolate productivity states)
    for j1 = 1:N_pr
        for i1 = 1:N_xi %match state
            aux = EE_H(:,:,i1,j1);
            EE_H_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value of H being employed today in H
            aux = EE_W(:,:,i1,j1);
            EE_W_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value W being employed today in W
            aux = EJ_H(:,:,i1,j1);
            EJ_H_H(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value J in H being employed today in H
            aux = EJ_W(:,:,i1,j1);
            EJ_W_W(:,:,i1,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value J in W being employed today in W
        end
        aux = EU(:,:,j1);
        EU_H1(:,:,j1) = (1-p_e)*aux+p_e*aux(index_H_1); %expected value U being H today
        EU_W1(:,:,j1) = (1-p_e)*aux+p_e*aux(index_W_1); %expected value U being W today
    end

    
    %expected value of unemployment when coming from employment given
    %redraw of preference
    EU_H = zeros(N_x,N_x,N_pr);
    EU_W = zeros(N_x,N_x,N_pr);
    for i = 1:N_pr %state
        for j= 1:N_pr %transition to
            EU_H(:,:,i) = EU_H(:,:,i)+tran_pr_new(i,j)*EU_H1(:,:,j);
            EU_W(:,:,i) = EU_W(:,:,i)+tran_pr_new(i,j)*EU_W1(:,:,j);
        end
    end
    
    %% Compute upper envelopes over expected values and resulting policy functions
    EU_L = repmat(reshape(EU,[N_x N_x 1 N_pr]),[1 1 N_xi 1]); %expand dimension of expected value of unemployment by match states
    EU_H_L = repmat(reshape(EU_H,[N_x N_x 1 N_pr]),[1 1 N_xi 1]); %expand dimension of expected value of unemployment by match states
    EU_W_L = repmat(reshape(EU_W,[N_x N_x 1 N_pr]),[1 1 N_xi 1]);
    
    %firm decision between continuing, layoff, ERTE 
    upper_F_RH_U = -psi(1)*ones(N_x,N_x,N_xi,N_pr); %upper envelop between ERTE H firm and layoff
    upper_F_H_N = max(EJ_H_H,upper_F_RH_U); %upper envelop between continuing a H firm and not
    upper_F_RW_U = -psi(2)*ones(N_x,N_x,N_xi,N_pr); %upper envelop between ERTE W firm and layoff
    upper_F_W_N = max(EJ_W_W,upper_F_RW_U); %upper envelop between continuing a W firm and not
    pol_F_RH_U = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer layoff)
    pol_F_H_N = upper_F_RH_U>EJ_H_H; %corresponding policy (prefer not to continue)
    pol_F_RW_U = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer layoff)
    pol_F_W_N = upper_F_RW_U>EJ_W_W; %corresponding policy (prefer not to continue)

    
    %recalling the worker from ERTE decision 
    upper_F_RH_H = EJ_H; %upper envelop between ERTE H firm and recalling worker 
    upper_F_RW_W = EJ_W; %upper envelop between ERTE W firm and recalling worker
    pol_F_RH_H = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer recall)
    pol_F_RW_W = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer recall)

    
    %decision of a unemployed searching worker to accept a potential job offer
    pol_U_H = EE_H>EU_L; %corresponding policy (prefer employment)
    pol_U_W = EE_W>EU_L; %corresponding policy (prefer employment)

    
    %continuation value in ERTE 
    Gamma_H = pol_F_RH_H.*EE_H+(1-pol_F_RH_H).*ER_H;
    Gamma_W = pol_F_RW_W.*EE_W+(1-pol_F_RW_W).*ER_W;
    
    %decision of a ERTE searching worker to accept a potential job offer 
    aux_GH = repmat(Gamma_H,[1 1 1 1 N_xi]); %extend by offer grid
    aux_EH = repmat(permute(EE_H,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer type in 6th dimension
    aux_EW = repmat(permute(EE_W,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer type in 6th dimension
    aux_GW = repmat(Gamma_W,[1 1 1 1 N_xi]); %extend by offer grid
    pol_RH_H = aux_EH>aux_GH; %corresponding policy (prefer employment)
    pol_RH_W = aux_EW>aux_GH; %corresponding policy (prefer employment)
    pol_RW_W = aux_EW>aux_GW; %corresponding policy (prefer employment)
    pol_RW_H = aux_EH>aux_GW; %corresponding policy (prefer employment)


    % Implied labor market tightness solving free entry equation 
    aux = max(pol_U_H,0.00001); %assure tightness is finite
    aux_vH = max(EJ_H,0);
    pol_F_fill_H = EJ_H>0; %policy whether to fill a vacancy
    theta_H_U = squeeze((eta(1)./sum((dis*chi(1)*aux.*aux_vH).*offer_U,3)).^(-1/gamma)); %labor market tightness in H for workers coming from U
    aux = max(pol_U_W,0.00001); %assure tightness is finite
    aux_vW = max(EJ_W,0);
    pol_F_fill_W = EJ_W>0; %policy whether to fill a vacancy
    theta_W_U = squeeze((eta(2)./sum((dis*chi(1)*aux.*aux_vW).*offer_U,3)).^(-1/gamma)); %labor market tightness in W for workers coming from U
    aux = max(pol_RH_H,0.00001); %assure tightness is finite
    aux2 = repmat(permute(aux_vH,[1 2 5 4 3]),[1 1 N_xi 1 1]); %value of the firm is independ on place where worker is right now
    theta_H_RH = squeeze((eta(1)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in H for workers coming from RH
    aux = max(pol_RW_H,0.00001); %assure tightness is finite
    theta_H_RW = squeeze((eta(1)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in H for workers coming from RW
    aux = max(pol_RH_W,0.00001); %assure tightness is finite
    aux2 = repmat(permute(aux_vW,[1 2 5 4 3]),[1 1 N_xi 1 1]); %value of the firm is independ on place where worker is right now
    theta_W_RH = squeeze((eta(2)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in W for workers coming from RH
    aux = max(pol_RW_W,0.00001); %assure tightness is finite
    theta_W_RW = squeeze((eta(2)./sum((dis*chi(2)*aux.*aux2).*offer_R,5)).^(-1/gamma)); %labor market tightness in W for workers coming from RW
    
    % Implied job offer rates (equation 5.14)
    p_H_U = min(max(chi(1)*theta_H_U.^(1-gamma),0.00001),1); %assure bounded probabilities
    p_H_RH = min(max(chi(2)*theta_H_RH.^(1-gamma),0.00001),1);
    p_H_RW = min(max(chi(2)*theta_H_RW.^(1-gamma),0.00001),1);
    p_W_U = min(max(chi(1)*theta_W_U.^(1-gamma),0.00001),1);
    p_W_RH = min(max(chi(2)*theta_W_RH.^(1-gamma),0.00001),1);
    p_W_RW = min(max(chi(2)*theta_W_RW.^(1-gamma),0.00001),1);
      

    %choice of worker in ERTE searching in particular sector 
    aux_EH = repmat(permute(pol_F_fill_H.*EE_H+(1-pol_F_fill_H).*Gamma_H,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer type in 6th dimension
    upper_RH_H = max(aux_EH,aux_GH); %upper envelop between staying ERTE H and going to H
    aux_EW = repmat(permute(pol_F_fill_W.*EE_W+(1-pol_F_fill_W).*Gamma_W,[1 2 5 4 3]),[1 1 N_xi 1 1]); %outside offer type in 6th dimension
    upper_RH_W = max(aux_EW,aux_GH); %upper envelop between staying ERTE H and going to W
    upper_RW_W = max(aux_EW,aux_GW); %upper envelop between staying ERTE W and going to W
    upper_RW_H = max(aux_EH,aux_GW); %upper envelop between staying ERTE W and going to W
    RS_HH = (1-p_H_RH).*Gamma_H+p_H_RH.*sum(upper_RH_H.*offer_R,5); %value of being in H and searching in H
    RS_HW = (1-p_W_RH).*Gamma_H+p_W_RH.*sum(upper_RH_W.*offer_R,5); %value of being in H and searching in W
    Lambda_H = max(RS_HH,RS_HW);
    RS_WH = (1-p_H_RW).*Gamma_W+p_H_RW.*sum(upper_RW_H.*offer_R,5); %value of being in W and searching in H
    RS_WW = (1-p_W_RW).*Gamma_W+p_W_RW.*sum(upper_RW_W.*offer_R,5); %value of being in W and searching in W
    Lambda_W = max(RS_WH,RS_WW);
    pol_RH_s = RS_HH>RS_HW; %corresponding policy (prefer H over W)
    pol_RW_s = RS_WW>RS_WH; %corresponding policy (prefer W over H)

    
    % Implied job leaving probability from ERTE
    pi_RH = pol_RH_s.*p_H_RH+(1-pol_RH_s).*p_W_RH;
    pi_RW = pol_RW_s.*p_W_RW+(1-pol_RW_s).*p_H_RW;
    
    %choice of worker in ERTE between staying in ERTE and unemployment
    upper_LU_H = Lambda_H;
    upper_LU_W = Lambda_W;
    pol_LU_H = ones(N_x,N_x,N_xi,N_pr);
    pol_LU_W = ones(N_x,N_x,N_xi,N_pr);
    
    
    %choice of unemployed between searching in sectors 
    upper_U_H = max(pol_F_fill_H.*EE_H+(1-pol_F_fill_H).*EU_L,EU_L); %upper envelop between staying unemployment and going to H
    upper_U_W = max(pol_F_fill_W.*EE_W+(1-pol_F_fill_W).*EU_L,EU_L); %upper envelop between staying unemployment and going to W
    US_H = (1-p_H_U).*EU+p_H_U.*squeeze(sum(upper_U_H.*offer_U,3));
    US_W = (1-p_W_U).*EU+p_W_U.*squeeze(sum(upper_U_W.*offer_U,3));
    upper_U_HW = max(US_H,US_W); %upper envelope
    pol_U_HW = US_H>US_W; %corresponding policy (prefer H)

    
    
    %Compute XI for workers using firm policies (equation 5.6)
    XI_H = (pol_F_H_N==1).*(pol_F_RH_U.*EU_H_L)...
        +(1-pol_F_H_N).*EE_H_H;
    XI_W = (pol_F_W_N==1).*(pol_F_RW_U.*EU_W_L)...
        +(1-pol_F_W_N).*EE_W_W;
    
    
    %quitting decision of the employed (part of equation 5.5)
    upper_XI_U_H = XI_H; %upper envelop between staying H and unemployment 
    upper_XI_U_W = XI_W; %upper envelop between staying W and unemployment 
    pol_XI_U_H=0; %always choose employment
    pol_XI_U_W=0;
    
    %% value functions
    discount = dis*(mu1);
    
    %value of unemployment
    U_new = b_U + discount*upper_U_HW; %value of unemployment 
    E_H_new = w_H_L + pr_grid_L + discount*(delta(1)*EU_L+(1-delta(1))*upper_XI_U_H); %value of employed in H 
    E_W_new = w_W_L - pr_grid_L + discount*(delta(2)*EU_L+(1-delta(2))*upper_XI_U_W); %value of employed in W 
    J_H_new = y_H_L-w_H_L -fc_H + discount*(1-delta(1))*(pol_XI_U_H==0).*upper_F_H_N; %value of filled job in H 
    J_W_new = y_W_L-w_W_L -fc_W + discount*(1-delta(2))*(pol_XI_U_W==0).*upper_F_W_N; %value of filled job in W 
    
    
    crit_J_H = max(max(max(max(max(abs(J_H_new-J_H))))));
    crit_J_W = max(max(max(max(max(abs(J_W_new-J_W))))));
    crit_E_H = max(max(max(max(max(abs(E_H_new-E_H))))));
    crit_E_W = max(max(max(max(max(abs(E_W_new-E_W))))));
    crit_U = max(max(max(max(max(abs(U_new-U))))));
    
    
    crit = max([crit_J_H crit_J_W crit_E_H crit_E_W crit_U]);
    
    %update
    J_H = J_H_new;
    J_W = J_W_new;
    E_H = E_H_new;
    E_W = E_W_new;
    U = U_new;
        
end

%recompute the policy functions
pol_F_RH_U = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer layoff)
pol_F_H_N = upper_F_RH_U>EJ_H_H; %corresponding policy (prefer not to continue)
pol_F_RW_U = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer layoff)
pol_F_W_N = upper_F_RW_U>EJ_W_W; %corresponding policy (prefer not to continue)
pol_F_RH_H = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer recall)
pol_F_RW_W = ones(N_x,N_x,N_xi,N_pr); %corresponding policy (prefer recall)
pol_U_H = EE_H>EU_L; %corresponding policy (prefer employment)
pol_U_W = EE_W>EU_L; %corresponding policy (prefer employment)
pol_RH_H = aux_EH>aux_GH; %corresponding policy (prefer employment)
pol_RH_W = aux_EW>aux_GH; %corresponding policy (prefer employment)
pol_RW_W = aux_EW>aux_GW; %corresponding policy (prefer employment)
pol_RW_H = aux_EH>aux_GW; %corresponding policy (prefer employment)
pol_RH_s = RS_HH>RS_HW; %corresponding policy (prefer H over W)
pol_RW_s = RS_WW>RS_WH; %corresponding policy (prefer W over H)
pol_U_HW = US_H>US_W; %corresponding policy (prefer H)
pol_XI_U_H = 0; %corresponding policy (prefer unemployment)
pol_XI_U_W = 0; %corresponding policy (prefer unemployment)



%save value and policy functions
steady.J_H = J_H;
steady.J_W = J_W;
steady.E_H = E_H;
steady.E_W = E_W;
steady.U = U;
steady.p_W_U = p_W_U;
steady.p_H_U = p_H_U;
steady.p_H_RH = p_H_RH;
steady.p_W_RH = p_W_RH;
steady.p_W_RW = p_W_RW;
steady.p_H_RW =p_H_RW;
steady.pol_U_HW=pol_U_HW;
steady.pol_U_W=pol_U_W;
steady.pol_F_fill_W=pol_F_fill_W;
steady.pol_U_H=pol_U_H;
steady.pol_F_fill_H=pol_F_fill_H;
steady.pol_F_H_N=pol_F_H_N;
steady.pol_F_RH_U=pol_F_RH_U;
steady.pol_F_W_N=pol_F_W_N;
steady.pol_F_RW_U=pol_F_RW_U;
steady.pol_RH_s=pol_RH_s;
steady.pol_RH_H=pol_RH_H;
steady.pol_F_RH_H=pol_F_RH_H;
steady.pol_RH_W=pol_RH_W;
steady.pol_RW_s=pol_RW_s;
steady.pol_RW_W=pol_RW_W;
steady.pol_F_RW_W=pol_F_RW_W;
steady.pol_RW_H=pol_RW_H;
save('steady.mat','steady')


%% compute moments in steady state

%use seeds
rand('seed',56424);
randn('seed',89726);

H = 30000; %number of people
T = 1000; %number of periods

%obtain shocks
ini_pr = rand(H,1); %initial preferences
die = rand(H,T); %surviving
offer = rand(H,T); %probability to get job offer 
offer_type_p = rand(H,T); %probability to get particular job offer 
destruct = rand(H,T); %probability to get unemployment exogenously
tran_xi = rand(H,T); %transition for match quality
skill_acc = rand(H,T); %probability to get one higher skill
shock_pr_new = rand(H,T); %probability to get a new preference
shock_pr = rand(H,T); %type of new preference


panel = zeros(H,10,T+1); %index, employment (U=0, H=1, W=2, RH=3, RW=4), xH, xW, xi, wage, switch jobs, taste, allive, search in sector


panel = simulation_mex(panel,int32(H),int32(T),int32(N_x),int32(N_xi),int32(N_pr),ini_pr,die,offer,offer_type_p ...
                    ,destruct,tran_xi,skill_acc,shock_pr_new,shock_pr,xi_prob,prob_pr_birth ...
                    ,mu,p_e,varphi,delta,Pxi_tran,w_H,w_W ...
                    ,p_W_U,p_H_U,int32(pol_U_HW),int32(pol_U_W),int32(pol_F_fill_W),int32(pol_U_H),int32(pol_F_fill_H) ...
                    ,int32(pol_F_H_N),int32(pol_F_RH_U),int32(pol_F_W_N),int32(pol_F_RW_U));


panel(:,:,end) = []; %drop last period

%collapse to 2d matrix
panel_new = zeros(H*(T),10);
for h = 1:H
    transform = squeeze(panel(h,:,1:end));
    transform = transform';
    panel_new(1+(h-1)*(T):h*(T),:) = transform;
end
panel = panel_new;
panel(panel(:,9)==0,:) = []; %delete the dead
clear panel_new

%save stationary distribution
panel_ss = panel(panel(:,1)<=2000,:); %use only first 2000 people
save('panel_ss.mat','panel_ss')

%age
panel(:,9) = 1;
fcn = @(x){cumsum(x)};
panel(:,9) = accumarray_expand(panel(:,9),panel(:,1),fcn); 


%index, employment (U=0, H=1, W=2, RH=3, RH=4), xH, xW, xi, wage, switch jobs
stats.unemp = sum(panel(:,2)==0)/length(panel);
stats.H_share = sum(panel(:,2)==1)/(sum(panel(:,2)==1)+sum(panel(:,2)==2));
stats.RH_share = sum(panel(:,2)==3)/length(panel);
stats.RW_share = sum(panel(:,2)==4)/length(panel);
aux = panel(panel(:,2)==1,6);
stats.wage_H = mean(aux);
aux = panel(panel(:,2)==2,6);
stats.wage_W = mean(aux);
stats.wage_diff = log(stats.wage_H)-log(stats.wage_W); 


%UE transition rate
aux = panel;
aux(:,10) = aux(:,2)>0; %single employment index
fcn = @(x){create_diff(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
fcn = @(x){create_LEAD(x)};
aux(:,12) = accumarray_expand(aux(:,2),aux(:,1),fcn); %next period's employment state
aux = aux(aux(:,2)==0,:); %keep the unemployed
aux(isnan(aux(:,11)),:) = []; %delete last observation
stats.UE = sum(aux(:,11)==1)./length(aux); %unconditional probability to find a job
aux(:,13) = cumsum(aux(:,12)); %unemployment spell
aux(2:end,13) =aux(1:end-1,13);
fcn = @(x){max(x)};
aux(:,12) = accumarray_expand(aux(:,12),aux(:,13),fcn); %copy next employment type to the entire unemploeymrnt spell
aux2 = aux(aux(:,12)==1,:); %unemployed who end up in sector H
stats.UE_H = sum(aux2(:,11)==1)./length(aux2);
aux2 = aux(aux(:,12)==2,:); %unemployed who end up in sector W
stats.UE_W = sum(aux2(:,11)==1)./length(aux2);


%EU transition rates
aux = panel;
aux(:,10) = aux(:,2)>0; %single employment index
fcn = @(x){create_diff(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
aux2 = aux(aux(:,2)==1,:); %employed in sector H
stats.EU_H = sum(aux2(:,11)==-1)./length(aux2);
aux2 = aux(aux(:,2)==2,:); %employed in sector W
stats.EU_W = sum(aux2(:,11)==-1)./length(aux2);


%tenure moments
aux = panel;
aux(:,10) = aux(:,2)>0; %single employment index (count ERTE as employed for calculating tenure)
fcn = @(x){create_diff2(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
aux(isnan(aux(:,11)),11) = 0;
aux(:,11) = aux(:,11) ==1;
fcn = @(x){cumsum(x)};
aux(:,12) = accumarray_expand(aux(:,11),aux(:,1),fcn); %number of job spells
fcn = @(x){create_diff(x)}; %employment ransition
aux(:,15) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
aux(:,15) = aux(:,15) ==-1; %become non-employed
aux(aux(:,10)==0,:) = []; %keep employed+ERTE
aux(:,13) = group([aux(:,1) aux(:,12)]); %individual job specific index
aux(:,14) = 1;
fcn = @(x){cumsum(x)};
aux(:,14) = accumarray_expand(aux(:,14),aux(:,13),fcn); %job tenure
stats.tenure.mean = mean(aux(:,14));
stats.tenure.median = median(aux(:,14));
stats.tenure.U.mean = mean(aux(aux(:,15)==1,14)); %tenure of those becoming unemployed
stats.tenure.U.median = median(aux(aux(:,15)==1,14));
aux2 = aux(aux(:,2)==1,:);
stats.tenure.median_H = median(aux2(:,14)); %of those in H
stats.tenure.mean_H = mean(aux2(:,14));
aux2 = aux(aux(:,2)==2,:);
stats.tenure.median_W = median(aux2(:,14));
stats.tenure.mean_W = mean(aux2(:,14));

%tenure distribution
[stats.tenure.density,stats.tenure.grid] = ksdensity(aux(:,14),linspace(1,max(aux(:,14)),max(aux(:,14))),'Bandwidth',1); 

%wage loss from EUE transitions, resulting std of log wage change, share of
%sector switchers from EUE
aux = panel;
aux(:,10) = aux(:,2)>0;%single employment index
fcn = @(x){cumsum(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %experience
aux(aux(:,11)==0,:) = []; %drop first unemployment spell
fcn = @(x){create_diff2(x)};
aux(:,11) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
aux(:,11) = aux(:,11)==-1; %make job loss an indicator
fcn = @(x){create_LAG(x)};
aux(:,12) = accumarray_expand(aux(:,6),aux(:,1),fcn); %lag log earnings
aux(:,13) = accumarray_expand(aux(:,5),aux(:,1),fcn); %lag job component
aux(:,14) = accumarray_expand(aux(:,2),aux(:,1),fcn); %lag sector
aux(:,15) = aux(:,11)~=0; %drop all observations until first job loss
fcn = @(x){cumsum(x)};
aux(:,15) = accumarray_expand(aux(:,15),aux(:,1),fcn); %transition spell
aux(aux(:,15)==0,:) = []; % drop everything until first job loss
fcn = @(x){create_diff2(x)};
aux(:,16) = accumarray_expand(aux(:,10),aux(:,1),fcn); %employment transitions
aux(:,16) = aux(:,16) ==1; %make job finding an indicator
aux(:,17) = aux(:,11)+aux(:,16); %job loss or job finding
aux(aux(:,17)==0,:) =[]; %keep only losses and findings
aux(:,18) = (aux(:,11)==1).*(aux(:,12)==0); %job loss out of ERTE
fcn = @(x){create_LAG(x)};
aux(:,19) = accumarray_expand(aux(:,18),aux(:,1),fcn); %job loss was due to ERTE
aux(isnan(aux(:,19)),19) = 0;
index = ((aux(:,18) ==1)+(aux(:,19) ==1))>0;
aux(index,:) = []; %delete those due to ERTE to U 
fcn = @(x){make_even(x)};
aux(:,20) = accumarray_expand(aux(:,9),aux(:,1),fcn); %last observation for off number of observations replaced by NaN
aux(isnan(aux(:,20)),:) = []; %drop last observation if resulting from EU
aux(:,6) = log(aux(:,6)); %log earnings
aux(:,12) = log(aux(:,12)); %lag log earnings
fcn = @(x){create_LAG(x)};
aux(:,12) = accumarray_expand(aux(:,12),aux(:,1),fcn); %lag log earnings
aux(:,13) = accumarray_expand(aux(:,13),aux(:,1),fcn); %lag job component
aux(:,14) = accumarray_expand(aux(:,14),aux(:,1),fcn); %lag sector
aux(:,20) = aux(:,6)-aux(:,12); %earnings growth
aux(:,21) = aux(:,2)~=aux(:,14); %sector switch
aux(:,22) = aux(:,5)-aux(:,13); %change in job component
aux(aux(:,6)==-Inf,:) = []; %keep only job finding observation
p_5 = prctile(aux(:,20),5);
p_95 = prctile(aux(:,20),95);
index = (aux(:,20)>p_5).*(aux(:,20)<p_95);
stats.EUE_std_change_bound = std(aux(index==1,20));
stats.EUE_share_sectro_switch = mean(aux(:,21)); %share of sector switchers
aux2 = aux(aux(:,2)==1,:); %workers in H sector today
EUE_wage.mean_entry_HH.all = mean(aux2(aux2(:,21)==0,6)); %mean log wage EUE H to H
aux2 = aux(aux(:,2)==2,:); %workers in W sector today
EUE_wage.mean_entry_HW.all = mean(aux2(aux2(:,21)==1,6)); %mean log wage EUE H to W



%share of unemployed searching in H 
aux = panel;
aux(:,11) = aux(:,2)>0;%single employment index
aux = aux(aux(:,11)==0,:);
aux(:,10) = aux(:,10)==1; %search in H
stats.share_search_H = mean(aux(:,10));

%average profit share
aux = panel(panel(:,2)>0,:);
aux(:,11) = aux(:,6)./0.95; %output
aux(aux(:,2)==1,6) = aux(aux(:,2)==1,6)+fc_H; %add fixed costs to wages
aux(aux(:,2)==2,6) = aux(aux(:,2)==2,6)+fc_W; %add fixed costs to wages
stats.profit_share = sum(aux(:,11)-aux(:,6))./sum(aux(:,11));

%calibration targets
cali.wage_W = stats.wage_W;
cali.wage_diff = stats.wage_diff;
cali.EUE_std_change_bound = stats.EUE_std_change_bound;
cali.EUE_HH_HW = EUE_wage.mean_entry_HH.all-EUE_wage.mean_entry_HW.all;
cali.EUE_share_sectro_switch = stats.EUE_share_sectro_switch;
cali.H_share = stats.H_share;
cali.UE = stats.UE;
cali.EU_H = stats.EU_H;
cali.EU_W = stats.EU_W;
cali.tenure_median = stats.tenure.median;


%% hypothetical tenure distribution with exogenous job destruction rates

H = 30000; %number of people
T = 1000; %number of periods

panel = zeros(H,3,T); %ID, sector, and tenure
panel(:,1,1) = 1:H;
index = round(stats.H_share*H); %share in H
panel(1:index,2,1) = 1; %H
panel(index+1:end,2,1) = 2; %W
panel(:,3,1) = 1; %start with 1 month of tenure


for t = 1:T-1
    panel(:,1,t) = 1:H;
    panel(1:index,2,t) = 1;
    panel(index+1:end,2,t) = 2;
    for i = 1:H
        if i<=index %person in H
            if (destruct(i,t)<=stats.EU_H) || (die(i,t)<=mu) %match end
                panel(i,3,t+1) = 1;
            else
                panel(i,3,t+1) = panel(i,3,t)+1;
            end
        else
            if (destruct(i,t)<=stats.EU_W) || (die(i,t)<=mu) %match end
                panel(i,3,t+1) = 1;
            else
                panel(i,3,t+1) = panel(i,3,t)+1;
            end
        end
    end
end

%collapse to 2d matrix
panel_new = zeros(H*(T),3);
for h = 1:H
    transform = squeeze(panel(h,:,1:end));
    transform = transform';
    panel_new(1+(h-1)*(T):h*(T),:) = transform;
end
panel = panel_new;
clear panel_new

[tenure_density_counter,tenure_grid_counter] = ksdensity(panel(:,3),linspace(1,max(panel(:,3)),max(panel(:,3))),'Bandwidth',1); 

%tenure in data
addpath('D:\Sectors\Clean\Data') 
tenure_dist_data = importdata('density_tenure.out');

plot(tenure_dist_data(1:200,1),tenure_dist_data(1:200,2)./sum(tenure_dist_data(:,2)),'LineWidth',2.5,'Color',[0 0 1])
hold on
plot(stats.tenure.grid(1:200),stats.tenure.density(1:200),'LineWidth',2.5,'LineStyle','--','Color',[1 0 0])
hold on
plot(tenure_grid_counter(1:200),tenure_density_counter(1:200),'LineWidth',2.5,'LineStyle',':','Color',[0 1 0])
legend('Data','Baseline model','Exogenous job destruction','FontSize',13)
hold off
ax = gca;
ax.FontSize = 16; 
xlabel('Month','fontsize',16)
ylabel('Density','fontsize',16)
width=700;
height=350;
set(gcf,'units','points','position',[0,0,width,height])

